﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Runtime.Serialization;
using System.Xml.Serialization;

using NuMetaheuristics;
using NuMetaheuristics.Genotypes;
using NuMetaheuristics.Operators.Vector;
using NuMetaheuristics.Operators.Value;
using NuMetaheuristics.Algorithms;
using NuMetaheuristics.Contexts;
using NuMetaheuristics.Utility;

namespace InitializationTest
{
    public class AlgorithmTesting
    {
        //public static void SerializeObjects(string contextXmlPath, string algorithmXmlPath)
        //{
        //    //
        //    // prepare the context
        //    //
            
        //    double min = 0.0;
        //    double max = 10.0;            
        //    var initializer = new VectorInitializer<double>(new DoubleRandomInitializer(min, max));

        //    var ideal = new DoubleVectorGenotype(54);
        //    initializer.Operate(ref ideal);

        //    var context = new DoubleVectorMatch(ideal, initializer);
        //    context.IdealFitness = new FitnessScoreSmallerIsBetter(1e-1);

        //    ContractSerializer.SerializeToXml<DoubleVectorMatch>(context, contextXmlPath);

        //    //
        //    // Prepare the algorithm
        //    //

        //    var mutator = new VectorMutator<double>(new DoubleGaussianConvolver(min, max, 0.002));
        //    var algorithm = new HillClimbingAlgorithm<DoubleVectorGenotype>(context, mutator, 35);

        //    ContractSerializer.SerializeToXml<HillClimbingAlgorithm<DoubleVectorGenotype>>(algorithm, algorithmXmlPath);
        //}

        //public static void RunFromXml(string contextXmlPath, string algorithmXmlPath)
        //{
        //    DoubleVectorMatch context = 
        //        ContractSerializer.DeserializeFromXml<DoubleVectorMatch>(contextXmlPath);
            
        //    HillClimbingAlgorithm<DoubleVectorGenotype> algorithm = 
        //        ContractSerializer.DeserializeFromXml<HillClimbingAlgorithm<DoubleVectorGenotype>>(algorithmXmlPath);

        //    DateTime start = System.DateTime.Now;

        //    var genome = context.ProduceGenome(true);
        //    Fitness fitness = (Fitness)context.Evaluator.EvaluateGenome(genome);
        //    Fitness bestFitness = (Fitness)fitness.Clone();

        //    bool bContinue = true;
        //    while (bContinue)
        //    {
        //        algorithm.Operate(ref genome);

        //        fitness = context.Evaluator.EvaluateGenome(genome);
        //        if (fitness > bestFitness)
        //        {
        //            bestFitness = (Fitness)fitness.Clone();
        //        }

        //        if (algorithm.Iteration % 50 == 0)
        //        {
        //            System.Console.WriteLine("fitness: {0:0.000}, best: {1:0.000}", fitness.Score, bestFitness.Score);
        //        }

        //        bContinue = (bestFitness < context.IdealFitness);
        //    }

        //    DateTime end = System.DateTime.Now;

        //    System.Console.WriteLine("Vector Length: {0}", genome.Vector.Length);
        //    System.Console.WriteLine("Iterations: {0}", algorithm.Iteration);
        //    System.Console.WriteLine("Computation Time: {0}", end.Subtract(start));
        //    System.Console.WriteLine("Best Fitness: {0}", bestFitness.Score);
        //}

        public static void AlgorithmTest<G>(IAlgorithm<G> algorithm, Population<G> population, IContext<G> context, Fitness idealFitness)
            where G : IGenotype
        {
            //var perturber = new DoubleVectorPerturber(new DoubleGaussianConvolver(min, max, 0.25), 4000);

            Fitness bestEver = (Fitness)population.Genomes.Last().Fitness.Clone();
            DateTime start = System.DateTime.Now;

            while (bestEver < idealFitness)
            {
                algorithm.Run(population, context, 25);
                population.Genomes.Sort();
                Fitness currentBest = population.Genomes.Last().Fitness;

                if (population.Genomes.Last().Fitness > bestEver)
                {
                    bestEver = (Fitness)currentBest.Clone();
                }

                System.Console.WriteLine("current best: {0:0.000}, best ever: {1:0.000}", currentBest, bestEver);
            }

            DateTime end = System.DateTime.Now;
            
            System.Console.WriteLine("Computation Time: {0}", end.Subtract(start));
            System.Console.WriteLine("Best Fitness: {0}", bestEver.Score);
        }

        public static void ParticleSwarmTest()
        {
            int length = 50; 
            double min = 0.0;
            double max = 10.0;
            
            double alpha = 0.4;
            double beta = 0.3;
            double gamma = 0.3;
            double delta = 0.3;
            double epsilon = 0.7;

            int swarmSize = 20;
            uint iterationsPer = 500;

            var positionInitializer = new DoubleVectorInitializer(new DoubleRandomInitializer(min, max));
            var positionBounding = new DoubleVectorBounding(min, max);
            var velocityInitializer = new DoubleVectorInitializer(new DoubleRandomInitializer(-1.0, 1.0));
            var velocityMutator = new DoubleVectorMutator(new DoubleGaussianConvolver(Double.MinValue, Double.MaxValue, .01));

            var ideal = new DoubleVectorGenotype(50);
            positionInitializer.Operate(ref ideal);

            Fitness idealFitness = new FitnessSmallerIsBetter(1e-1);
            var context = new DoubleVectorMatch(length, min, max);

            var alg = new ParticleSwarmAlgorithm<DoubleVectorGenotype>(
                velocityInitializer, 
                velocityMutator,
                new RandomDoubleVectorScalarMultiplier(new DoubleRandomInitializer(0.0, alpha)),
                new RandomDoubleVectorScalarMultiplier(new DoubleRandomInitializer(0.0, beta)),
                new RandomDoubleVectorScalarMultiplier(new DoubleRandomInitializer(0.0, gamma)),
                new RandomDoubleVectorScalarMultiplier(new DoubleRandomInitializer(0.0, delta)),
                new RandomDoubleVectorScalarMultiplier(new DoubleRandomInitializer(0.0, epsilon)),
                new DoubleVectorAdder(),
                new DoubleVectorSubtracter()
                );

            var population = new Population<DoubleVectorGenotype>(swarmSize, context);
            population.Genomes.Sort();
            
            Fitness bestFitness = (Fitness)population.Genomes.Last().Fitness.Clone();
            
            long startSec = System.DateTime.Now.Second;
            bool bContinue = true;
            while (bContinue)
            {
                alg.Run(population, context, iterationsPer);
                population.Genomes.Sort();
                bestFitness = (Fitness)population.Genomes.Last().Fitness.Clone();
                System.Console.WriteLine("fitness: {0:0.000}", bestFitness.Score);
                bContinue = (bestFitness < idealFitness);
            }

            long endSec = System.DateTime.Now.Second;

            System.Console.WriteLine("Vector Length: {0}", ideal.Vector.Length);
            System.Console.WriteLine("Computation Time: {0} seconds", endSec - startSec);
            System.Console.WriteLine("Best Fitness: {0}", bestFitness.Score);
        }
    }
}
