﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Runtime.Serialization;

using NuMetaheuristics;
using NuMetaheuristics.Utility;
using NuMetaheuristics.Algorithms;
using NuMetaheuristics.Genotypes;
using NuMetaheuristics.Operators.Vector;
using NuMetaheuristics.Operators.Value;
using NuMetaheuristics.Contexts;

using NuMetaheuristics.Contexts.Vector.Boolean;

namespace InitializationTest
{
    class Program
    {
        static void Main(string[] args)
        {
            {
                IContext<DoubleVectorGenotype> context = new DoubleVectorMatch(4, 0.0, 1.0);

                var serializer = new ContractSerializer<IContext<DoubleVectorGenotype>>();
                Type[] knownTypes = new Type[]
                {
                    typeof(DoubleVectorMatch),
                    typeof(DoubleVectorGenotype),
                    typeof(DoubleVectorInitializer)
                };

                serializer.Serialize(context, "problemSpace.xml", knownTypes);
                var problemSpace2 = serializer.Deserialize("problemSpace.xml", knownTypes);
            }

            {
                int maxItemsPerSlot = 100;

                var weights = new DoubleVectorGenotype(new double[] { 0.5, 1.5, 1.9, 3.0, 3.8, 4.9, 3.0, 5.0 });
                var values = new DoubleVectorGenotype(new double[] { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0 });

                IContext<IntegerVectorGenotype> context = new Knapsack(values.Vector.Length, maxItemsPerSlot, values, weights, weights.Vector.Sum() * 2.0);
                IBinaryOperator<IntegerVectorGenotype> crossover = new IntegerVectorUniformCrossover();
                IUnaryOperator<IntegerVectorGenotype> mutator = new IntegerVectorMutator(new IntegerGaussianConvolver(0, maxItemsPerSlot, (double)maxItemsPerSlot / 8));

                var population = new Population<IntegerVectorGenotype>(100, context);
                var algorithm = new GeneticAlgorithm<IntegerVectorGenotype>(population.Genomes.Count() / 4, crossover, mutator);

                Fitness idealFitness = new FitnessBiggerIsBetter(120);
                AlgorithmTesting.AlgorithmTest<IntegerVectorGenotype>(algorithm, population, context, idealFitness);
            }

            AlgorithmTesting.ParticleSwarmTest();

            //var genome = new DoubleVectorGenotype(20);
            //double min = 0.0;
            //double max = 10.0;

            //var initializer = new VectorInitializer<double>(new DoubleRandomInitializer(min, max));
            //initializer.Operate(ref genome);

            ////ContractSerializer.SerializeToXml<DoubleVectorGenotype>(genome, "genome.xml");
            ////IGenotype<DoubleVectorGenotype> genome2 = ContractSerializer.DeserializeFromXml<DoubleVectorGenotype>("genome.xml");

            //var context = new DoubleVectorMatch(genome, initializer);

            //var genome3 = new DoubleVectorGenotype(20);
            //initializer.Operate(ref genome3);

            //context.Evaluator.EvaluateGenome(genome3);
        }
    }
}
