﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Operators.Value;

using Xunit;

namespace NuMetaheuristics.UnitTests.Operators.Value
{
    public class IntegerAdderTest
    {
        private void subroutine(IntegerValueAdder x, int a, int b)
        {
            int result1 = x.Operate(a, b);
            int result2 = a + b;

            Assert.True(result1 == result2);
        }

        public void multTest()
        {
            var x = new IntegerValueAdder();

            subroutine(x, 1, 4);
            subroutine(x, 2, 4);
            subroutine(x, 12, 17);
            subroutine(x, 29, 533);
            subroutine(x, 92, 441);
            subroutine(x, 29, -28);
            subroutine(x, 100, -3005);
            subroutine(x, Int32.MinValue, Int32.MinValue);
            subroutine(x, Int32.MaxValue, Int32.MaxValue);
        }

        [Fact]
        public void BasicTest()
        {
            Assert.DoesNotThrow(multTest);
        }
    }

    public class DoubleAdderTest
    {
        private void subroutine(DoubleValueAdder x, double a, double b)
        {
            double result1 = x.Operate(a, b);
            double result2 = a + b;

            Assert.True(result1 == result2);
        }

        public void multTest()
        {
            var x = new DoubleValueAdder();

            subroutine(x, 1, 4);
            subroutine(x, 1.11, 4.096);
            subroutine(x, 2, 44e11);
            subroutine(x, 12, -17.5);
            subroutine(x, 292.2, 53.3);
            subroutine(x, 9.01e22, 4.41);
            subroutine(x, 29e-9, -28.0);
            subroutine(x, Double.MaxValue, Double.MaxValue);
            subroutine(x, Double.MaxValue, Double.MinValue);
            subroutine(x, Double.MaxValue, Double.MaxValue);
        }

        [Fact]
        public void BasicTest()
        {
            multTest();
        }
    }

    public class IntegerSubtractorTest
    {
        private void subroutine(IntegerValueSubtracter x, int a, int b)
        {
            int result2 = x.Operate(a, b); 
            int result1 = a - b;

            Assert.True(result1 == result2);
        }

        public void divTest()
        {
            var x = new IntegerValueSubtracter();

            subroutine(x, 0, 1);
            subroutine(x, 4, 4);
            subroutine(x, 4, 1);
            subroutine(x, 4, 2);
            subroutine(x, 1, 4);
            subroutine(x, 2, 4);
            subroutine(x, 17, 3);
            subroutine(x, 533, 29);
            subroutine(x, 441, 92);
            subroutine(x, 299, -28);
            subroutine(x, -3005, 100);
            subroutine(x, Int32.MinValue, Int32.MinValue);
            subroutine(x, Int32.MaxValue, Int32.MaxValue);
        }

        [Fact]
        public void BasicTest()
        {
            Assert.DoesNotThrow(divTest);
        }
    }

    public class DoubleSubtractorTest
    {
        private void subroutine(DoubleValueSubtracter x, double a, double b)
        {
            double result2 = x.Operate(a, b);
            double result1 = a - b;

            Assert.True(result1 == result2);
        }

        public void divTest()
        {
            var x = new DoubleValueSubtracter();

            subroutine(x, 0.0, 4);
            subroutine(x, 1, 4);
            subroutine(x, 1.11, 4.096);
            subroutine(x, 2, 44e11);
            subroutine(x, 12, -17.5);
            subroutine(x, 292.2, 53.3);
            subroutine(x, 9.01e22, 4.41);
            subroutine(x, 29e-9, -28.0);
            subroutine(x, Double.MinValue, Double.MinValue);
            subroutine(x, Double.MaxValue, Double.MinValue);
            subroutine(x, Double.MaxValue, Double.MaxValue);
        }

        [Fact]
        public void BasicTest()
        {
            Assert.DoesNotThrow(divTest);
        }
    }
}
