﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Operators.Value;

using Xunit;

namespace NuMetaheuristics.UnitTests.Operators.Value
{
    public class IntegerGaussianConvTest
    {
        public void subroutine(int min, int max, int stdDev)
        {
            var x = new IntegerRandomInitializer(-10, 10);
            var y = new IntegerGaussianConvolver(-10, 10, 1);

            for (int i = 0; i < 40; i++)
            {
                int z = 0;
                x.Operate(ref z);
                y.Operate(ref z);

                Assert.True(z >= min);
                Assert.True(z <= max);
            }
        }

        [Fact]
        public void BasicRange()
        {
            subroutine(-10, 10, 1);
        }

        [Fact]
        public void ExtremeRange()
        {
            subroutine(Int32.MinValue, Int32.MaxValue, Int32.MaxValue / 10);
        }

        [Fact]
        public void LargeFluctuationBasic()
        {
            subroutine(-10, 10, 5);
        }

        [Fact]
        public void LargeFluctuationExtreme()
        {
            subroutine(Int32.MinValue, Int32.MaxValue, Int32.MaxValue/ 5);
        }
    }

    public class DoubleGaussianConvTest
    {
        public void subroutine(double min, double max, double stdDev)
        {
            var x = new DoubleRandomInitializer(-10, 10);
            var y = new DoubleGaussianConvolver(-10, 10, 1);

            for (int i = 0; i < 40; i++)
            {
                double z = 0;
                x.Operate(ref z);
                y.Operate(ref z);

                Assert.True(z >= min);
                Assert.True(z <= max);
            }
        }

        [Fact]
        public void BasicRange()
        {
            subroutine(-10.0, 10.0, 1.0);
        }

        [Fact]
        public void ExtremeRange()
        {
            subroutine(Double.MinValue, Double.MaxValue, Double.MaxValue / 10.0 );
        }

        [Fact]
        public void LargeFluctuationBasic()
        {
            subroutine(-10.0, 10.0, 5.0);
        }

        [Fact]
        public void LargeFluctuationExtreme()
        {
            subroutine(Double.MinValue, Double.MaxValue, Double.MaxValue / 5.0);
        }
    }
}