﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Operators.Value;

using Xunit;

namespace NuMetaheuristics.UnitTests.Operators.Value
{
    public class IntegerMultiplierTest
    {
        private void multTestSub(IntegerMultiplier x, int a, int b)
        {
            int result1 = x.Operate(a, b);
            int result2 = a * b;

            Assert.True(result1 == result2);
        }

        public void multTest()
        {
            var x = new IntegerMultiplier();

            multTestSub(x, 1, 4);
            multTestSub(x, 2, 4);
            multTestSub(x, 12, 17);
            multTestSub(x, 29, 533);
            multTestSub(x, 92, 441);
            multTestSub(x, 29, -28);
            multTestSub(x, 100, -3005);
            multTestSub(x, Int32.MinValue, Int32.MinValue);
            multTestSub(x, Int32.MaxValue, Int32.MaxValue);
        }

        [Fact]
        public void BasicTest()
        {
            Assert.DoesNotThrow(multTest);
        }
    }

    public class DoubleMultiplierTest
    {
        private void multTestSub(DoubleMultiplier x, double a, double b)
        {
            double result1 = x.Operate(a, b);
            double result2 = a * b;

            Assert.True(result1 == result2);
        }

        public void multTest()
        {
            var x = new DoubleMultiplier();

            multTestSub(x, 1, 4);
            multTestSub(x, 1.11, 4.096);
            multTestSub(x, 2, 44e11);
            multTestSub(x, 12, -17.5);
            multTestSub(x, 292.2, 53.3);
            multTestSub(x, 9.01e22, 4.41);
            multTestSub(x, 29e-9, -28.0);
            multTestSub(x, Double.MaxValue, Double.MaxValue);
            multTestSub(x, Double.MaxValue, Double.MinValue);
            multTestSub(x, Double.MaxValue, Double.MaxValue);
        }

        [Fact]
        public void BasicTest()
        {
            multTest();
        }
    }

    public class IntegerDividerTest
    {
        private void divTestSub(IntegerDivider x, int a, int b)
        {
            int result1 = a / b;
            int result2 = x.Operate(a, b);

            Assert.True(result1 == result2);
        }

        public void divTest()
        {
            var x = new IntegerDivider();

            divTestSub(x, 0, 1);
            divTestSub(x, 4, 4);
            divTestSub(x, 4, 1);
            divTestSub(x, 4, 2);
            divTestSub(x, 1, 4);
            divTestSub(x, 2, 4);
            divTestSub(x, 17, 3);
            divTestSub(x, 533, 29);
            divTestSub(x, 441, 92);
            divTestSub(x, 299, -28);
            divTestSub(x, -3005, 100);
            divTestSub(x, Int32.MinValue, Int32.MinValue);
            divTestSub(x, Int32.MaxValue, Int32.MaxValue);
        }

        public void divByZeroTest()
        {
            var x = new IntegerDivider();

            divTestSub(x, 1, 0);
            divTestSub(x, 4, 0);
            divTestSub(x, 1000, 0);
            divTestSub(x, -1000, 0);
        }

        [Fact]
        public void BasicTest()
        {
            Assert.DoesNotThrow(divTest);
        }

        [Fact]
        public void DivideByZeroTest()
        {
            Assert.ThrowsDelegate x = new Assert.ThrowsDelegate(divByZeroTest);
            Assert.Throws<DivideByZeroException>(x);
        }
    }

    public class DoubleDividerTest
    {
        private void divTestSub(DoubleDivider x, double a, double b)
        {
            double result2 = x.Operate(a, b);
            double result1 = a / b;

            Assert.True(result1 == result2);
        }

        public void divTest()
        {
            var x = new DoubleDivider();

            divTestSub(x, 0.0, 4);
            divTestSub(x, 1, 4);
            divTestSub(x, 1.11, 4.096);
            divTestSub(x, 2, 44e11);
            divTestSub(x, 12, -17.5);
            divTestSub(x, 292.2, 53.3);
            divTestSub(x, 9.01e22, 4.41);
            divTestSub(x, 29e-9, -28.0);
            divTestSub(x, Double.MinValue, Double.MinValue);
            divTestSub(x, Double.MaxValue, Double.MinValue);
            divTestSub(x, Double.MaxValue, Double.MaxValue);
            divTestSub(x, 10.0, 0.0);
        }

        [Fact]
        public void BasicTest()
        {
            Assert.DoesNotThrow(divTest);
        }
    }
}
