﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Operators.Value;

using Xunit;

namespace NuMetaheuristics.UnitTests.Operators.Value
{
    public class IntegerRandomInitializerTest
    {
        private int _min;
        private int _max;

        private void intRangeTest()
        {
            IntegerRandomInitializer x = new IntegerRandomInitializer(_min, _max);

            for (int i = 0; i < 40; i++)
            {
                int y = 0;
                x.Operate(ref y);
                Assert.True(y >= x.Min);
                Assert.True(y <= x.Max);
            }
        }

        [Fact]
        public void IntegerRangeDefault()
        {
            var x = new IntegerRandomInitializer();
            _min = x.Min;
            _max = x.Max;
            Assert.DoesNotThrow(intRangeTest);
        }

        [Fact]
        public void IntegerRange1()
        {
            _min = -5000;
            _max = 0;
            Assert.DoesNotThrow(intRangeTest);
        }

        [Fact]
        public void IntegerRange2()
        {
            _min = 0;
            _max = 5000;
            Assert.DoesNotThrow(intRangeTest);
        }
    }

    public class DoubleRandomInitializerTest
    {
        private double _min;
        private double _max;

        private void doubleRangeTest()
        {
            DoubleRandomInitializer x = new DoubleRandomInitializer(_min, _max);

            for (int i = 0; i < 40; i++)
            {
                double y = 0;
                x.Operate(ref y);
                Assert.True(y >= x.Min);
                Assert.True(y <= x.Max);
            }
        }

        [Fact]
        public void DoubleRangeDefault()
        {
            var x = new DoubleRandomInitializer();
            _min = x.Min;
            _max = x.Max;
            Assert.DoesNotThrow(doubleRangeTest);
        }

        [Fact]
        public void DoubleRange1()
        {
            _min = -5000;
            _max = 0;
            Assert.DoesNotThrow(doubleRangeTest);
        }

        [Fact]
        public void DoubleRange2()
        {
            _min = 0;
            _max = 5000;
            Assert.DoesNotThrow(doubleRangeTest);
        }
    }

    public class BooleanRandomInitializerTest
    {
        private void booleanRangeTest()
        {
            var x = new BooleanRandomInitializer();
            for (int i = 0; i < 40; i++)
            {
                bool y = false;
                x.Operate(ref y);
            }
        }

        [Fact]
        public void BooleanAssertTest()
        {
            Assert.DoesNotThrow(booleanRangeTest);
        }
    }
}
