﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Genotypes;
using NuMetaheuristics.Operators.Value;
using NuMetaheuristics.Operators.Vector;

using Xunit;

namespace NuMetaheuristics.UnitTests
{
    public class VectorBoundTests
    {
        [Fact]
        public void IntBoundTest1()
        {
            var x = new int[3] { 1, 2, 3 };
            var y = new int[3] { 0, 0, 0 };
            var bounding = new IntegerVectorBounding(1, 3);

            var adder = new IntegerVectorAdder();
            var result = adder.Operate(new IntegerVectorGenotype(x), new IntegerVectorGenotype(y));
            bounding.Operate(ref result);

            Assert.True(result.Vector.SequenceEqual(x));
        }

        [Fact]
        public void IntBoundTest2()
        {
            var x = new int[3] { 1, 2, 3 };
            var y = new int[3] { 1, 1, 1 };
            var z = new int[3] { 2, 3, 3 };
            var bounding = new IntegerVectorBounding(1, 3);

            var adder = new IntegerVectorAdder();
            var result = adder.Operate(new IntegerVectorGenotype(x), new IntegerVectorGenotype(y));
            bounding.Operate(ref result);

            Assert.True(result.Vector.SequenceEqual(z));
        }

        [Fact]
        public void IntBoundTest3()
        {
            var x = new int[3] { 1, 2, 3 };
            var y = new int[3] { -1, -1, -1 };
            var z = new int[3] { 1, 1, 2 };
            var bounding = new IntegerVectorBounding(1, 3);

            var adder = new IntegerVectorAdder();
            var result = adder.Operate(new IntegerVectorGenotype(x), new IntegerVectorGenotype(y));
            bounding.Operate(ref result);

            Assert.True(result.Vector.SequenceEqual(z));
        }

        [Fact]
        public void DblBoundTest1()
        {
            var x = new double[3] { 1.0, 2.0, 3.0 };
            var y = new double[3] { 0.0, 0.0, 0.0 };

            var bounding = new DoubleVectorBounding(1.0, 3.0);

            var adder = new DoubleVectorAdder();
            var result = adder.Operate(new DoubleVectorGenotype(x), new DoubleVectorGenotype(y));
            bounding.Operate(ref result);

            Assert.True(result.Vector.Contains(1.0) && result.Vector.Contains(2.0) && result.Vector.Contains(3.0));
        }

        [Fact]
        public void DblBoundTest2()
        {
            var x = new double[3] { 1.0, 2.0, 3.0 };
            var y = new double[3] { 1.0, 1.0, 1.0 };
            var bounding = new DoubleVectorBounding(1.0, 3.0);

            var adder = new DoubleVectorAdder();
            var result = adder.Operate(new DoubleVectorGenotype(x), new DoubleVectorGenotype(y));
            bounding.Operate(ref result);

            Assert.True(result.Vector.Contains(2.0) && result.Vector.Contains(3.0) && !result.Vector.Contains(4.0));
        }

        [Fact]
        public void DblBoundTest3()
        {
            var x = new double[3] { 1.0, 2.0, 3.0 };
            var y = new double[3] { -1.0, -1.0, -1.0 };
            var bounding = new DoubleVectorBounding(1.0, 3.0);

            var adder = new DoubleVectorAdder();
            var result = adder.Operate(new DoubleVectorGenotype(x), new DoubleVectorGenotype(y));
            bounding.Operate(ref result);

            Assert.True(!result.Vector.Contains(0.0) && result.Vector.Contains(1.0) && result.Vector.Contains(2.0));
        }
    }
}
