﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Genotypes;
using NuMetaheuristics.Operators.Value;
using NuMetaheuristics.Operators.Vector;

using Xunit;

namespace NuMetaheuristics.UnitTests.Vector
{
    public class VectorCrossoverTests
    {
        [Fact]
        public void CrossoverDoubleVector()
        {
            const double min = 2.0;
            const double max = 4.0;
            var initializer = new DoubleVectorInitializer(new DoubleRandomInitializer(min, max));
            var a = new DoubleVectorGenotype(20);
            var b = new DoubleVectorGenotype(20);

            initializer.Operate(ref a);
            initializer.Operate(ref b);

            var crossover = new DoubleVectorUniformCrossover();
            DoubleVectorGenotype child = new DoubleVectorGenotype(crossover.Operate(a, b));

            Assert.True(child.Vector.Length == a.Vector.Length || child.Vector.Length == b.Vector.Length);
        }

        public void CrossoverDoubleVectorDifferentLegths()
        {
            const double min = 2.0;
            const double max = 4.0;
            var initializer = new DoubleVectorInitializer(new DoubleRandomInitializer(min, max));
            var a = new DoubleVectorGenotype(15);
            var b = new DoubleVectorGenotype(8);

            initializer.Operate(ref a);
            initializer.Operate(ref b);

            var crossover = new DoubleVectorUniformCrossover();
            DoubleVectorGenotype child = new DoubleVectorGenotype(crossover.Operate(a, b));

            Assert.True(child.Vector.Length == a.Vector.Length || child.Vector.Length == b.Vector.Length);
        }
    }
}
