﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NuMetaheuristics;
using NuMetaheuristics.Genotypes;
using NuMetaheuristics.Operators.Value;
using NuMetaheuristics.Operators.Vector;

using Xunit;

namespace NuMetaheuristics.UnitTests.Vector
{
    public class VectorMutationTests
    {
        [Fact]
        public void MutationDoubleVector()
        {
            const double min = 2.0;
            const double max = 4.0;
            var initializer = new DoubleVectorInitializer(new DoubleRandomInitializer(min, max));
            var a = new DoubleVectorGenotype(20);

            initializer.Operate(ref a);

            var mutator = new DoubleVectorMutator(new DoubleGaussianConvolver(min, max, 0.5));
            mutator.Operate(ref a);

            Assert.True(a.Vector.Min() >= min && a.Vector.Max() <= max);
        }

        public void MutationIntVector()
        {
            const int min = -50;
            const int max = 50;
            var initializer = new IntegerVectorInitializer(new IntegerRandomInitializer(min, max));
            var a = new IntegerVectorGenotype(20);

            initializer.Operate(ref a);

            var mutator = new IntegerVectorMutator(new IntegerGaussianConvolver(min, max, 5.0));
            mutator.Operate(ref a);

            Assert.True(a.Vector.Min() >= min && a.Vector.Max() <= max);
        }
    }
}
