﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Xunit;

using NuMetaheuristics;
using NuMetaheuristics.Genotypes;
using NuMetaheuristics.Contexts.Vector.Boolean;

namespace NuMetaheuristics.UnitTests
{
    public class PopulationTest
    {
        public void checkOrdering<G>(Population<G> x) where G : IGenotype
        {
            Fitness worst= x.Genomes.First().Fitness;
            Fitness best = x.Genomes.Last().Fitness;

            foreach(IGenotype genome in x.Genomes)
            {
                if (genome.Fitness < worst)
                {
                    worst = genome.Fitness;
                }

                if (genome.Fitness > best)
                {
                    best = genome.Fitness;
                }
            }

            Assert.True(worst.Equals(x.Genomes.First().Fitness));
            Assert.True(best.Equals(x.Genomes.Last().Fitness));
        }
        
        [Fact]
        public void DefaultCtorTest()
        {
            var x = new Population<BooleanVectorGenotype>();

            Assert.True(x.Genomes != null);
            Assert.True(x.Genomes.Count == 0);
        }

        [Fact]
        public void FullCtorTest()
        {
            var x = new Population<BooleanVectorGenotype>(10, new MaxOnes(4));

            Assert.True(x.Genomes != null);
            Assert.True(x.Genomes.Count == 10);

            checkOrdering<BooleanVectorGenotype>(x);
        }

        [Fact]
        public void InsertWorst()
        {
            var context = new MaxOnes(8);
            var x = new Population<BooleanVectorGenotype>(10, context);
            var worst = new BooleanVectorGenotype(new bool[] { false, false, false, false, false, false, false, false });

            context.Evaluate(ref worst);
            x.Genomes.Add(worst);
            x.Genomes.Sort();

            checkOrdering<BooleanVectorGenotype>(x);
        }

        [Fact]
        public void InsertBest()
        {
            var context = new MaxOnes(8);
            var x = new Population<BooleanVectorGenotype>(10, context);
            var best = new BooleanVectorGenotype(new bool[] { true, true, true, true, true, true, true, true });

            context.Evaluate(ref best);
            x.Genomes.Add(best);
            x.Genomes.Sort();

            checkOrdering<BooleanVectorGenotype>(x);
        }
    }
}
